# Python Version: 3.x
import abc
import pathlib
from typing import Any, Optional, Sequence

import competitive_verifier_oj_clone.languages.special_comments as special_comments


class LanguageEnvironment:
    def get_compile_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> Optional[str]:
        """
        :throws Exception:
        """
        return None

    @abc.abstractmethod
    def get_execute_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        raise NotImplementedError


class Language:
    def list_attributes(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> dict[str, Any]:
        """
        :throws Exception:
        """

        attributes: dict[str, Any] = special_comments.list_special_comments(path)
        attributes.setdefault("links", [])
        attributes["links"].extend(special_comments.list_embedded_urls(path))
        return attributes

    @abc.abstractmethod
    def list_dependencies(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> list[pathlib.Path]:
        """
        :throws Exception:
        """

        raise NotImplementedError

    def bundle(self, path: pathlib.Path, *, basedir: pathlib.Path) -> Optional[bytes]:
        """
        :throws Exception:
        """
        return None

    def is_verification_file(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> bool:
        return ".test." in path.name

    @abc.abstractmethod
    def list_environments(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> Sequence[LanguageEnvironment]:
        raise NotImplementedError



import pathlib
from datetime import datetime, timedelta, timezone
from json import dumps as json_dumps
from typing import Any

import pytest

from competitive_verifier.models import (
    FileResult,
    ResultStatus,
    VerificationResult,
    VerifyCommandResult,
)

test_parse_FileResult_params = [  # type: ignore
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2016, 12, 24, 15, 16, 34),
                )
            ],
        ),
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": "2016-12-24 15:16:34",
                }
            ],
            "newest": True,
        },
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": datetime(2016, 12, 24, 15, 16, 34),
                }
            ],
            "newest": True,
        },
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": "2016-12-24T15:16:34",
                }
            ],
            "newest": True,
        },
    ),
    (
        FileResult(
            newest=False,
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(
                        2016, 12, 24, 15, 16, 34, tzinfo=timezone.utc
                    ),
                )
            ],
        ),
        {
            "newest": False,
            "verifications": [
                {
                    "status": "Success",
                    "elapsed": 1.5,
                    "last_execution_time": "2016-12-24 15:16:34Z",
                }
            ],
        },
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": datetime(
                        2016, 12, 24, 15, 16, 34, tzinfo=timezone.utc
                    ),
                }
            ],
            "newest": False,
        },
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": "2016-12-24T15:16:34+00:00",
                }
            ],
            "newest": False,
        },
    ),
    (
        FileResult(
            newest=True,
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(
                        2016, 12, 24, 15, 16, 34, tzinfo=timezone(timedelta(hours=9))
                    ),
                )
            ],
        ),
        {
            "verifications": [
                {
                    "elapsed": 1.5,
                    "status": "SUCCESS",
                    "last_execution_time": "2016-12-24 15:16:34+09:00",
                }
            ]
        },
        {
            "verifications": [
                {
                    "elapsed": 1.5,
                    "status": "success",
                    "last_execution_time": datetime(
                        2016, 12, 24, 15, 16, 34, tzinfo=timezone(timedelta(hours=9))
                    ),
                }
            ],
            "newest": True,
        },
        {
            "verifications": [
                {
                    "status": "success",
                    "elapsed": 1.5,
                    "last_execution_time": "2016-12-24T15:16:34+09:00",
                }
            ],
            "newest": True,
        },
    ),
]


@pytest.mark.parametrize(
    "obj, raw_dict, output_dict, output_json_dict",
    test_parse_FileResult_params,
)
def test_parse_FileResult(
    obj: FileResult,
    raw_dict: dict[str, Any],
    output_dict: dict[str, Any],
    output_json_dict: dict[str, Any],
):
    assert obj == FileResult.parse_obj(raw_dict)
    assert obj.dict() == output_dict
    assert obj.json() == json_dumps(output_json_dict)


test_file_result_need_verification_params = [  # type: ignore
    (FileResult(verifications=[]), datetime(2016, 12, 24, 19, 0, 0), False),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2019, 12, 24, 19, 0, 0),
                )
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        False,
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.FAILURE,
                    last_execution_time=datetime(2019, 12, 24, 19, 0, 0),
                )
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        True,
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SKIPPED,
                    last_execution_time=datetime(2019, 12, 24, 19, 0, 0),
                )
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        True,
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2015, 12, 24, 19, 0, 0),
                )
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        True,
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2018, 12, 24, 19, 0, 0),
                ),
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2018, 12, 24, 19, 0, 0),
                ),
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2015, 12, 24, 19, 0, 0),
                ),
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        True,
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2018, 12, 24, 19, 0, 0),
                ),
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.SUCCESS,
                    last_execution_time=datetime(2018, 12, 24, 19, 0, 0),
                ),
                VerificationResult(
                    elapsed=1.5,
                    status=ResultStatus.FAILURE,
                    last_execution_time=datetime(2018, 12, 24, 19, 0, 0),
                ),
            ]
        ),
        datetime(2016, 12, 24, 19, 0, 0),
        True,
    ),
]


@pytest.mark.parametrize(
    "obj, dt, expected",
    test_file_result_need_verification_params,
)
def test_file_result_need_verification(
    obj: FileResult,
    dt: datetime,
    expected: bool,
):
    assert obj.need_verification(dt) == expected


test_is_success_params = [
    (
        FileResult(
            verifications=[
                VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
                VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
            ]
        ),
        (True, True),
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
                VerificationResult(elapsed=1, status=ResultStatus.FAILURE),
            ]
        ),
        (False, False),
    ),
    (
        FileResult(
            verifications=[
                VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
                VerificationResult(elapsed=1, status=ResultStatus.SKIPPED),
            ]
        ),
        (False, True),
    ),
]


@pytest.mark.parametrize(
    "obj, expected",
    test_is_success_params,
)
def test_is_success(
    obj: FileResult,
    expected: tuple[bool, bool],
):
    assert obj.is_success(allow_skip=False) == expected[0]
    assert obj.is_success(allow_skip=True) == expected[1]


def test_verify_command_result_json():
    obj = VerifyCommandResult(
        total_seconds=3.75,
        files={
            pathlib.Path("foo/bar.py"): FileResult(
                verifications=[
                    VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
                    VerificationResult(elapsed=1, status=ResultStatus.SKIPPED),
                ]
            ),
            pathlib.Path("foo/baz.py"): FileResult(
                verifications=[
                    VerificationResult(elapsed=1, status=ResultStatus.SUCCESS),
                ]
            ),
        },
    )
    assert VerifyCommandResult.parse_raw(obj.json()) == obj



from datetime import timedelta

import pytest

from competitive_verifier.summary import to_human_str

test_to_human_str_params: list[tuple[timedelta, str]] = [
    (timedelta(hours=2, minutes=8, seconds=6, milliseconds=13), "2h 8m"),
    (timedelta(minutes=8, seconds=6, milliseconds=13), "8m 6s"),
    (timedelta(seconds=16, milliseconds=13), "16s"),
    (timedelta(seconds=6, milliseconds=3), "6.0s"),
    (timedelta(seconds=6, milliseconds=13), "6.0s"),
    (timedelta(seconds=6, milliseconds=213), "6.2s"),
    (timedelta(milliseconds=1), "1ms"),
    (timedelta(milliseconds=12), "12ms"),
    (timedelta(milliseconds=123), "123ms"),
]


@pytest.mark.parametrize(
    "td, expected",
    test_to_human_str_params,
)
def test_to_human_str(td: timedelta, expected: str):
    assert to_human_str(td.total_seconds()) == expected



# Python Version: 3.x
import functools
import os
import pathlib
import platform
import shutil
from logging import getLogger
from typing import Any, Optional

import competitive_verifier_oj_clone.languages.special_comments as special_comments
import competitive_verifier_oj_clone.shlex2 as shlex
from competitive_verifier_oj_clone.config import get_config
from competitive_verifier_oj_clone.languages.cplusplus_bundle import Bundler
from competitive_verifier_oj_clone.languages.models import Language, LanguageEnvironment

from .. import subprocess2 as subprocess

logger = getLogger(__name__)


class CPlusPlusLanguageEnvironment(LanguageEnvironment):
    CXX: pathlib.Path
    CXXFLAGS: list[str]

    def __init__(self, *, CXX: pathlib.Path, CXXFLAGS: list[str]):
        self.CXX = CXX  # type:ignore
        self.CXXFLAGS = CXXFLAGS  # type:ignore

    def get_compile_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        return shlex.join(
            [
                str(self.CXX),
                *self.CXXFLAGS,
                "-I",
                str(basedir),
                "-o",
                str(tempdir / "a.out"),
                str(path),
            ]
        )

    def get_execute_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        return str(tempdir / "a.out")

    def is_clang(self) -> bool:
        return "clang++" in self.CXX.name

    def is_gcc(self) -> bool:
        return not self.is_clang() and "g++" in self.CXX.name


@functools.lru_cache(maxsize=None)
def _cplusplus_list_depending_files(
    path: pathlib.Path, *, CXX: pathlib.Path, joined_CXXFLAGS: str
) -> list[pathlib.Path]:
    is_windows = platform.uname().system == "Windows"
    command = [str(CXX), *shlex.split(joined_CXXFLAGS), "-MM", str(path)]
    try:
        data = subprocess.run(command, text=False).stdout
    except Exception:
        logger.error(
            "failed to analyze dependencies with %s: %s  (hint: Please check #include directives of the file and its dependencies. The paths must exist, must not contain '\\', and must be case-sensitive.)",
            CXX,
            str(path),
        )
        raise
    logger.debug("dependencies of %s: %s", str(path), repr(data))
    makefile_rule = shlex.split(
        data.decode().strip().replace("\\\n", "").replace("\\\r\n", ""),
        posix=not is_windows,
    )
    return [pathlib.Path(path).resolve() for path in makefile_rule[1:]]


@functools.lru_cache(maxsize=None)
def _cplusplus_list_defined_macros(
    path: pathlib.Path, *, CXX: pathlib.Path, joined_CXXFLAGS: str
) -> dict[str, str]:
    command = [str(CXX), *shlex.split(joined_CXXFLAGS), "-dM", "-E", str(path)]
    data = subprocess.run(command, text=False).stdout
    define: dict[str, str] = {}
    for line in data.decode().splitlines():
        assert line.startswith("#define ")
        a, _, b = line[len("#define ") :].partition(" ")
        if (b.startswith('"') and b.endswith('"')) or (
            b.startswith("'") and b.endswith("'")
        ):
            b = b[1:-1]
        define[a] = b
    return define


_NOT_SPECIAL_COMMENTS = "*NOT_SPECIAL_COMMENTS*"
_PROBLEM = "PROBLEM"
_IGNORE = "IGNORE"
_IGNORE_IF_CLANG = "IGNORE_IF_CLANG"
_IGNORE_IF_GCC = "IGNORE_IF_GCC"
_ERROR = "ERROR"


# config.toml example:
#     [[languages.cpp.environments]]
#     CXX = "g++"
#     CXXFALGS = ["-std=c++17", "-Wall"]
class CPlusPlusLanguage(Language):
    config: dict[str, Any]

    def __init__(self, *, config: Optional[dict[str, Any]] = None):
        if config is None:
            self.config = get_config()["languages"].get("cpp", {})
        else:
            self.config = config

    def _list_environments(self) -> list[CPlusPlusLanguageEnvironment]:
        default_CXXFLAGS = ["--std=c++17", "-O2", "-Wall", "-g"]
        if platform.system() == "Windows" or "CYGWIN" in platform.system():
            default_CXXFLAGS.append("-Wl,-stack,0x10000000")
        if platform.system() == "Darwin":
            default_CXXFLAGS.append("-Wl,-stack_size,0x10000000")
        if (
            platform.uname().system == "Linux"
            and "Microsoft" in platform.uname().release
        ):
            default_CXXFLAGS.append("-fsplit-stack")

        if "CXXFLAGS" in os.environ and "environments" not in self.config:
            logger.warning(
                "Usage of $CXXFLAGS envvar to specify options is deprecated and will be removed soon"
            )
            default_CXXFLAGS = shlex.split(os.environ["CXXFLAGS"])

        envs: list[CPlusPlusLanguageEnvironment] = []
        if "environments" in self.config:
            # configured: use specified CXX & CXXFLAGS
            for env in self.config["environments"]:
                CXX: str = env.get("CXX")
                if CXX is None:
                    raise RuntimeError("CXX is not specified")
                CXXFLAGS: list[str] = env.get("CXXFLAGS", default_CXXFLAGS)
                if not isinstance(CXXFLAGS, list):  # type:ignore
                    raise RuntimeError("CXXFLAGS must ba a list")
                envs.append(
                    CPlusPlusLanguageEnvironment(
                        CXX=pathlib.Path(CXX), CXXFLAGS=CXXFLAGS
                    )
                )

        elif "CXX" in os.environ:
            # old-style: 以前は $CXX を使ってたけど設定ファイルに移行したい
            logger.warning(
                "Usage of $CXX envvar to restrict compilers is deprecated and will be removed soon"
            )
            envs.append(
                CPlusPlusLanguageEnvironment(
                    CXX=pathlib.Path(os.environ["CXX"]), CXXFLAGS=default_CXXFLAGS
                )
            )

        else:
            # default: use found compilers
            for name in ("g++", "clang++"):
                path = shutil.which(name)
                if path is not None:
                    envs.append(
                        CPlusPlusLanguageEnvironment(
                            CXX=pathlib.Path(path), CXXFLAGS=default_CXXFLAGS
                        )
                    )

        if not envs:
            raise RuntimeError("No C++ compilers found")
        return envs

    def list_attributes(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> dict[str, Any]:
        attributes: dict[str, Any] = {}

        comments = special_comments.list_special_comments(path.resolve())
        if comments:
            attributes.update(comments)

        elif super().is_verification_file(path, basedir=basedir):
            # use old-style if special comments not found
            # #define PROBLEM "https://..." の形式は複数 environments との相性がよくない。あと遅い
            attributes[_NOT_SPECIAL_COMMENTS] = ""
            all_ignored = True
            for env in self._list_environments():
                joined_CXXFLAGS = " ".join(
                    map(shlex.quote, [*env.CXXFLAGS, "-I", str(basedir)])
                )
                macros = _cplusplus_list_defined_macros(
                    path.resolve(), CXX=env.CXX, joined_CXXFLAGS=joined_CXXFLAGS
                )

                # convert macros to attributes
                if _IGNORE not in macros:
                    for key in [_PROBLEM, _ERROR]:
                        if all_ignored:
                            # the first non-ignored environment
                            if key in macros:
                                attributes[key] = macros[key]
                        else:
                            assert attributes.get(key) == macros.get(key)
                    all_ignored = False
                else:
                    if env.is_gcc():
                        attributes[_IGNORE_IF_GCC] = ""
                    elif env.is_clang():
                        attributes[_IGNORE_IF_CLANG] = ""
                    else:
                        attributes[_IGNORE] = ""
            if all_ignored:
                attributes[_IGNORE] = ""

        attributes.setdefault("links", [])
        attributes["links"].extend(special_comments.list_embedded_urls(path))
        return attributes

    def list_dependencies(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> list[pathlib.Path]:
        env = self._list_environments()[0]
        joined_CXXFLAGS = " ".join(
            map(shlex.quote, [*env.CXXFLAGS, "-I", str(basedir)])
        )
        return _cplusplus_list_depending_files(
            path.resolve(), CXX=env.CXX, joined_CXXFLAGS=joined_CXXFLAGS
        )

    def bundle(self, path: pathlib.Path, *, basedir: pathlib.Path) -> Optional[bytes]:
        include_paths: list[pathlib.Path] = [basedir]
        assert isinstance(include_paths, list)
        bundler = Bundler(iquotes=include_paths)
        bundler.update(path)
        return bundler.get()

    def list_environments(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> list[CPlusPlusLanguageEnvironment]:
        attributes = self.list_attributes(path, basedir=basedir)
        envs: list[CPlusPlusLanguageEnvironment] = []
        for env in self._list_environments():
            if env.is_gcc() and _IGNORE_IF_GCC in attributes:
                continue
            if env.is_clang() and _IGNORE_IF_CLANG in attributes:
                continue
            envs.append(env)
        return envs



import argparse
import logging
import sys
from logging import getLogger
from typing import Optional

from competitive_verifier.log import configure_logging

from . import migration

logger = getLogger(__name__)


def run_impl(dry_run: bool) -> bool:
    return migration.main(dry_run)


def run(args: argparse.Namespace) -> bool:
    logger.debug("arguments=%s", vars(args))
    return run_impl(args.dry_run)


def argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    parser.add_argument(
        "--dry-run",
        "-n",
        dest="dry_run",
        action="store_true",
        help="Run a trial migration with no changes. Just show logs only.",  # noqa: E501
    )
    return parser


def main(args: Optional[list[str]] = None) -> None:
    try:
        configure_logging(logging.INFO)
        parsed = argument(argparse.ArgumentParser()).parse_args(args)
        if not run(parsed):
            sys.exit(1)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(2)


if __name__ == "__main__":
    main()



import fnmatch
import os
import pathlib
import traceback
from itertools import chain
from logging import getLogger
from typing import Generator

import competitive_verifier.config as config
import competitive_verifier.git as git
import competitive_verifier.oj as oj
import competitive_verifier_oj_clone.list
from competitive_verifier.models import (
    AddtionalSource,
    ConstVerification,
    ProblemVerification,
    ResultStatus,
    Verification,
    VerificationFile,
    VerificationInput,
)
from competitive_verifier_oj_clone.languages.models import LanguageEnvironment

logger = getLogger(__name__)


def get_bundled_dir() -> pathlib.Path:
    return config.config_dir / "bundled"


class OjResolver:
    include: list[str]
    exclude: list[str]
    _match_exclude_cache: dict[pathlib.Path, bool]

    def __init__(
        self,
        *,
        include: list[str],
        exclude: list[str],
    ) -> None:
        self.include = include
        self.exclude = exclude
        self._match_exclude_cache = {}

    def _match_exclude2(self, paths: list[pathlib.Path]) -> bool:
        if not paths:
            return False
        path = paths.pop()
        cache = self._match_exclude_cache.get(path, None)
        if cache is not None:
            return cache

        for ex in self.exclude:
            if fnmatch.fnmatch(path.as_posix(), ex):
                self._match_exclude_cache[path] = True
                return True
        result = self._match_exclude2(paths)
        self._match_exclude_cache[path] = result
        return result

    def _match_exclude(self, path: pathlib.Path) -> bool:
        paths = list(path.parents)
        paths.reverse()
        paths.append(path)
        return self._match_exclude2(paths)

    def resolve(self, *, bundle: bool) -> VerificationInput:
        files: dict[pathlib.Path, VerificationFile] = {}
        basedir = pathlib.Path.cwd()

        for path in git.ls_files(*self.include):
            if self._match_exclude(path):
                logger.debug("exclude=%s", path.as_posix())
                continue

            language = competitive_verifier_oj_clone.list.get(path)
            if language is None:
                continue

            deps = set(git.ls_files(*language.list_dependencies(path, basedir=basedir)))
            attr = language.list_attributes(path, basedir=basedir)

            def env_to_verifications(
                env: LanguageEnvironment,
            ) -> Generator[Verification, None, None]:
                if "IGNORE" in attr:
                    yield ConstVerification(status=ResultStatus.SKIPPED)
                    return

                error_str = attr.get("ERROR")
                error = float(error_str) if error_str else None

                tle_str = attr.get("TLE")
                tle = float(tle_str) if tle_str else None

                url = attr.get("PROBLEM")

                if url:
                    tempdir = oj.get_directory(url)
                    yield ProblemVerification(
                        command=env.get_execute_command(
                            path, basedir=basedir, tempdir=tempdir
                        ),
                        compile=env.get_compile_command(
                            path, basedir=basedir, tempdir=tempdir
                        ),
                        problem=url,
                        error=error,
                        tle=tle,
                    )

                unit_test_envvar = attr.get("UNITTEST")
                if unit_test_envvar:
                    var = os.getenv(unit_test_envvar)
                    if var is None:
                        logger.warning(
                            "UNITTEST envvar %s is not defined.",
                            unit_test_envvar,
                        )
                        yield ConstVerification(status=ResultStatus.FAILURE)
                    elif var.lower() == "false" or var == "0":
                        logger.info(
                            "UNITTEST envvar %s=%s is falsy.",
                            unit_test_envvar,
                            var,
                        )
                        yield ConstVerification(status=ResultStatus.FAILURE)
                    else:
                        logger.info(
                            "UNITTEST envvar %s=%s is truthy.",
                            unit_test_envvar,
                            var,
                        )
                        yield ConstVerification(status=ResultStatus.SUCCESS)

            additonal_sources: list[AddtionalSource] = []
            if bundle:
                try:
                    bundled_code = language.bundle(path, basedir=basedir)
                    if bundled_code:
                        dest_dir = get_bundled_dir()
                        dest_path = dest_dir / path
                        dest_path.parent.mkdir(parents=True, exist_ok=True)
                        logger.info("bundle_path=%s", dest_path.as_posix())
                        dest_path.write_bytes(bundled_code)
                        additonal_sources.append(
                            AddtionalSource(name="bundled", path=dest_path)
                        )
                except Exception:
                    bundled_code = traceback.format_exc()
                    dest_dir = get_bundled_dir()
                    dest_path = dest_dir / path
                    dest_path.parent.mkdir(parents=True, exist_ok=True)
                    logger.info("bundle_path=%s", dest_path.as_posix())
                    dest_path.write_text(bundled_code)
                    additonal_sources.append(
                        AddtionalSource(name="bundle error", path=dest_path)
                    )

            verifications = list(
                chain.from_iterable(
                    env_to_verifications(vs)
                    for vs in language.list_environments(path, basedir=basedir)
                )
            )
            files[path] = VerificationFile(
                dependencies=deps,
                verification=verifications,
                document_attributes=attr,
                additonal_sources=additonal_sources,
            )
        return VerificationInput(files=files)



import pathlib
from functools import cached_property
from logging import getLogger
from typing import Any, Optional, Union

from pydantic import BaseModel, Field, StrBytes, validator

from competitive_verifier.util import to_relative

from ._scc import SccGraph
from .verification import Verification

logger = getLogger(__name__)

_DependencyEdges = dict[pathlib.Path, set[pathlib.Path]]


class AddtionalSource(BaseModel):
    name: str
    """Source type
    """
    path: pathlib.Path
    """Source file path
    """


class VerificationFile(BaseModel):
    dependencies: set[pathlib.Path] = Field(default_factory=set)
    verification: list[Verification] = Field(default_factory=list)
    document_attributes: dict[str, Any] = Field(default_factory=dict)
    """Attributes for documentation
    """
    additonal_sources: list[AddtionalSource] = Field(default_factory=list)
    """Addtional source paths
    """

    class Config:
        json_encoders = {pathlib.Path: lambda v: v.as_posix()}  # type: ignore

    @validator("verification", pre=True)
    def verification_list(cls, v: Any) -> list[Any]:
        if v is None:
            return []
        elif isinstance(v, list):
            return v  # type: ignore
        return [v]

    def is_verification(self) -> bool:
        return bool(self.verification)

    def is_skippable_verification(self) -> bool:
        """
        If verification cost is small, it is skippable.
        """
        return self.is_verification() and all(v.is_skippable for v in self.verification)


# NOTE: computed fields  https://github.com/pydantic/pydantic/pull/2625
class VerificationInputImpl(BaseModel):
    files: dict[pathlib.Path, VerificationFile] = Field(default_factory=dict)

    def json(self, **kwargs: Any) -> str:
        class WithStrDict(BaseModel):
            files: dict[str, VerificationFile] = Field(default_factory=dict)

            class Config:
                json_encoders = {pathlib.Path: lambda v: v.as_posix()}  # type: ignore

        return WithStrDict(
            files={k.as_posix(): v for k, v in self.files.items()},
        ).json(**kwargs)


class VerificationInput:
    impl: VerificationInputImpl

    def __init__(
        self,
        impl: Optional[VerificationInputImpl] = None,
        *,
        files: Optional[dict[pathlib.Path, VerificationFile]] = None,
    ) -> None:
        if not impl:
            impl = VerificationInputImpl(
                files=files,  # type: ignore
            )
        self.impl = impl

    def merge(self, other: "VerificationInput") -> "VerificationInput":
        return VerificationInput(files=self.files | other.files)

    @property
    def files(self) -> dict[pathlib.Path, VerificationFile]:
        return self.impl.files

    def __repr__(self) -> str:
        return "VerificationInput" + repr(self.impl)[len("VerificationInputImpl") :]

    def __eq__(self, other: object) -> bool:
        if isinstance(other, VerificationInput):
            return self.impl == other.impl
        elif isinstance(other, VerificationInputImpl):
            return self.impl == other
        return NotImplemented

    def dict(self, **kwargs: Any) -> dict[str, Any]:
        return self.impl.dict(**kwargs)

    def json(self, **kwargs: Any) -> str:
        return self.impl.json(**kwargs)

    @staticmethod
    def parse_raw(b: StrBytes, **kwargs: Any) -> "VerificationInput":
        return VerificationInput(VerificationInputImpl.parse_raw(b, **kwargs))

    @staticmethod
    def parse_file_relative(
        path: Union[str, pathlib.Path], **kwargs: Any
    ) -> "VerificationInput":
        impl = VerificationInputImpl.parse_file(path, **kwargs)
        new_files: dict[pathlib.Path, VerificationFile] = {}
        for p, f in impl.files.items():
            p = to_relative(p)
            if not p:
                continue
            f.dependencies = set(d for d in map(to_relative, f.dependencies) if d)
            new_files[p] = f

        impl.files = new_files
        return VerificationInput(impl)

    @staticmethod
    def parse_obj(obj: Any) -> "VerificationInput":
        return VerificationInput(VerificationInputImpl.parse_obj(obj))

    def scc(self, *, reversed: bool = False) -> list[set[pathlib.Path]]:
        """Strongly Connected Component

        Args:
            reversed bool: if True, libraries are ahead. otherwise, tests are ahead.

        Returns:
            list[set[pathlib.Path]]: Strongly Connected Component result
        """
        paths = list(p for p in self.files.keys())
        vers_rev = {v: i for i, v in enumerate(paths)}
        g = SccGraph(len(paths))
        for p, file in self.files.items():
            for e in file.dependencies:
                t = vers_rev.get(e, -1)
                if 0 <= t:
                    if reversed:
                        g.add_edge(t, vers_rev[p])
                    else:
                        g.add_edge(vers_rev[p], t)
        return [set(paths[ix] for ix in ls) for ls in g.scc()]

    @cached_property
    def transitive_depends_on(self) -> _DependencyEdges:
        d: _DependencyEdges = {}
        g = self.scc(reversed=True)
        for group in g:
            result = group.copy()
            for p in group:
                for dep in self.files[p].dependencies:
                    if dep not in result:
                        resolved = d.get(dep)
                        if resolved is not None:
                            result.update(resolved)
            for p in group:
                d[p] = result

        return d

    def _build_dependency_graph(
        self,
    ) -> tuple[_DependencyEdges, _DependencyEdges, _DependencyEdges]:
        """
        Returns: Dependency graphs
        """
        depends_on: _DependencyEdges = {}
        required_by: _DependencyEdges = {}
        verified_with: _DependencyEdges = {}

        # initialize
        for path in self.files.keys():
            depends_on[path] = set()
            required_by[path] = set()
            verified_with[path] = set()

        # build the graph
        for src, vf in self.files.items():
            for dst in vf.dependencies:
                if src == dst:
                    continue
                if dst not in depends_on:
                    msg = (
                        "The file `%s` which is depended from `%s` is ignored "
                        "because it's not listed as a source code file."
                    )
                    logger.warning(msg, dst, src)
                    continue

                depends_on[src].add(dst)
                if vf.is_verification():
                    verified_with[dst].add(src)
                else:
                    required_by[dst].add(src)

        return depends_on, required_by, verified_with

    def _init_dependency_graph(self) -> None:
        (
            self._depends_on,
            self._required_by,
            self._verified_with,
        ) = self._build_dependency_graph()

    @property
    def depends_on(self) -> _DependencyEdges:
        try:
            d = self._depends_on
        except AttributeError:
            self._init_dependency_graph()
            d = self._depends_on
        return d

    @property
    def required_by(self) -> _DependencyEdges:
        try:
            d = self._required_by
        except AttributeError:
            self._init_dependency_graph()
            d = self._required_by
        return d

    @property
    def verified_with(self) -> _DependencyEdges:
        try:
            d = self._verified_with
        except AttributeError:
            self._init_dependency_graph()
            d = self._verified_with
        return d



from abc import ABC, abstractmethod
from typing import Annotated, Literal, Optional, Protocol, Union

from pydantic import BaseModel, Field

from .. import oj
from ..exec import exec_command
from .result_status import ResultStatus


class VerificationParams(Protocol):
    default_tle: float


class BaseVerification(BaseModel, ABC):
    @abstractmethod
    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        ...

    @abstractmethod
    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        ...

    @property
    def is_skippable(self) -> bool:
        """
        If verification cost is small, it is skippable.
        """
        return False


class ConstVerification(BaseVerification):
    type: Literal["const"] = "const"
    status: ResultStatus

    @property
    def is_skippable(self) -> bool:
        return True

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        return self.status

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        return True


class CommandVerification(BaseVerification):
    type: Literal["command"] = "command"

    command: str
    compile: Optional[str] = None

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        if exec_command(self.command, text=True).returncode == 0:
            return ResultStatus.SUCCESS
        return ResultStatus.FAILURE

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        if self.compile:
            return exec_command(self.compile, text=True).returncode == 0
        return True


class ProblemVerification(BaseVerification):
    type: Literal["problem"] = "problem"

    command: str
    compile: Optional[str] = None

    problem: str
    """
    problem: URL of problem
    """

    error: Optional[float] = None
    tle: Optional[float] = None

    def run(
        self,
        params: Optional[VerificationParams] = None,
    ) -> ResultStatus:
        if not params:
            raise ValueError("ProblemVerification.run requires VerificationParams")

        if oj.test(
            url=self.problem,
            command=self.command,
            tle=self.tle or params.default_tle,
            error=self.error,
        ):
            return ResultStatus.SUCCESS
        return ResultStatus.FAILURE

    def run_compile_command(
        self,
        params: Optional[VerificationParams] = None,
    ) -> bool:
        if self.compile:
            return exec_command(self.compile, text=True).returncode == 0
        return True


Verification = Annotated[
    Union[
        ConstVerification,
        CommandVerification,
        ProblemVerification,
    ],
    Field(discriminator="type"),
]



import pathlib
from typing import Any, Optional

from pydantic import BaseModel


class FrontMatter(BaseModel):
    title: Optional[str] = None
    layout: Optional[str] = None
    documentation_of: Optional[str] = None
    data: Optional[dict[str, Any]] = None
    redirect_from: Optional[list[str]] = None
    """for jekyll-redirect-from plugin
    """

    def merge(self, other: "FrontMatter") -> "FrontMatter":
        result = self.copy()
        if not result.title:
            result.title = other.title
        if not result.layout:
            result.layout = other.layout
        if not result.documentation_of:
            result.documentation_of = other.documentation_of
        if not result.redirect_from:
            result.redirect_from = other.redirect_from

        if not result.data:
            result.data = other.data
        elif other.data:
            result.data |= other.data
        return result


class PageRenderJob(BaseModel):
    path: pathlib.Path
    """jekyll markdown path

    relative path from basedir
    """
    document_path: Optional[pathlib.Path]
    """original markdown path

    relative path from basedir
    """

    front_matter: FrontMatter
    content: bytes

    def merge(self, other: "PageRenderJob") -> "PageRenderJob":
        assert self.path == other.path
        result = self.copy()
        if not result.document_path:
            result.document_path = other.document_path
        if not result.content:
            result.content = other.content
        result.front_matter = result.front_matter.merge(other.front_matter)
        return result


class SiteRenderConfig(BaseModel):
    static_dir: pathlib.Path
    index_md: pathlib.Path
    destination_dir: pathlib.Path
    config_yml: dict[str, Any]



import argparse
import pathlib
import sys
from typing import Optional

import competitive_verifier_oj_clone.config

from .resolver import OjResolver


def run_impl(
    include: list[str],
    exclude: list[str],
    config_path: Optional[pathlib.Path],
    enable_bundle: bool,
) -> bool:
    if config_path:
        competitive_verifier_oj_clone.config.set_config_path(config_path)

    resolver = OjResolver(
        include=include,
        exclude=exclude,
    )
    resolved = resolver.resolve(bundle=enable_bundle)
    print(resolved.impl.json(exclude_none=True))
    return True


def run(args: argparse.Namespace) -> bool:
    return run_impl(
        include=args.include,
        exclude=args.exclude,
        config_path=args.config,
        enable_bundle=args.bundle,
    )


def argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    parser.add_argument(
        "--include",
        nargs="*",
        help="Included file",
        default=[],
        type=str,
    )
    parser.add_argument(
        "--exclude",
        nargs="*",
        help="Excluded file",
        default=[],
        type=str,
    )
    parser.add_argument(
        "--no-bundle",
        dest="bundle",
        action="store_false",
        help="Disable bundle",
    )
    parser.add_argument(
        "--config",
        help="config.toml",
        type=pathlib.Path,
    )

    return parser


def main(args: Optional[list[str]] = None) -> None:
    try:
        parsed = argument(argparse.ArgumentParser()).parse_args(args)
        if not run(parsed):
            sys.exit(1)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(2)


if __name__ == "__main__":
    main()



import argparse
import importlib.metadata
import sys
from logging import DEBUG, INFO, getLogger
from typing import Callable, Optional

logger = getLogger(__name__)


def get_parser() -> argparse.ArgumentParser:
    import competitive_verifier.check.main as check
    import competitive_verifier.documents.main as docs
    import competitive_verifier.download.main as download
    import competitive_verifier.merge_input.main as merge_input
    import competitive_verifier.merge_result.main as merge_result
    import competitive_verifier.migrate.main as migrate
    import competitive_verifier.oj_resolve.main as oj_resolve
    import competitive_verifier.verify.main as verify

    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", action="store_true")
    parser.add_argument(
        "--version",
        action="store_true",
        help="print the online-judge-tools version number",
    )

    subparsers = parser.add_subparsers(dest="subcommand")

    subparser = subparsers.add_parser(
        "verify",
        help="Verify library",
    )
    verify.argument(subparser)

    subparser = subparsers.add_parser(
        "docs",
        help="Create documents",
    )
    docs.argument(subparser)

    subparser = subparsers.add_parser(
        "download",
        help="Download problems",
    )
    download.argument(subparser)

    subparser = subparsers.add_parser(
        "merge-input",
        help="Merge verify_files.json",
    )
    merge_input.argument(subparser)

    subparser = subparsers.add_parser(
        "merge-result",
        help="Merge result of `verify`",
    )
    merge_result.argument(subparser)

    subparser = subparsers.add_parser(
        "check",
        help="Check result of `verify`",
    )
    check.argument(subparser)

    subparser = subparsers.add_parser(
        "oj-resolve",
        help="Create verify_files json using `oj-verify`",
    )
    oj_resolve.argument(subparser)

    subparser = subparsers.add_parser(
        "migrate",
        help="Migration from verification-helper(`oj-verify`) project",
    )
    migrate.argument(subparser)

    return parser


def select_logging_runner(
    subcommand: str,
) -> tuple[Callable[[int], None], Optional[Callable[[argparse.Namespace], bool]]]:
    import competitive_verifier.check.main as check
    import competitive_verifier.documents.main as docs
    import competitive_verifier.download.main as download
    import competitive_verifier.merge_input.main as merge_input
    import competitive_verifier.merge_result.main as merge_result
    import competitive_verifier.migrate.main as migrate
    import competitive_verifier.oj_resolve.main as oj_resolve
    import competitive_verifier.verify.main as verify
    from competitive_verifier.log import configure_logging, configure_stderr_logging

    # Use sys.stdout for result
    if subcommand == "merge-result":
        return configure_stderr_logging, merge_result.run
    if subcommand == "merge-input":
        return configure_stderr_logging, merge_input.run
    if subcommand == "oj-resolve":
        return configure_stderr_logging, oj_resolve.run
    if subcommand == "check":
        return configure_stderr_logging, check.run
    if subcommand == "migrate":
        return configure_stderr_logging, migrate.run

    # Use sys.stdout for logging
    if subcommand == "download":
        return configure_logging, download.run
    if subcommand == "verify":
        return configure_logging, verify.run
    if subcommand == "docs":
        return configure_logging, docs.run
    return configure_logging, None


def main(args: Optional[list[str]] = None):

    parser = get_parser()
    parsed = parser.parse_args(args)

    if parsed.version:
        print(importlib.metadata.version("competitive-verifier"))
        sys.exit(0)

    default_level = INFO
    if parsed.verbose:
        default_level = DEBUG

    configure_log, runner = select_logging_runner(parsed.subcommand)

    configure_log(default_level)
    if runner:
        sys.exit(0 if runner(parsed) else 1)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import yaml

from .type import FrontMatter

_separator: bytes = b"---"


def _split_front_matter_without_parsing_yaml(content: bytes) -> tuple[bytes, bytes]:
    lines = content.splitlines(keepends=True)
    if len(lines) == 0 or lines[0].rstrip() != _separator:
        return (b"", content)
    for i, line in enumerate(lines):
        if i == 0:
            continue
        if line.rstrip() == _separator:
            break
    else:
        return b"", content

    front_matter = b"".join(lines[1:i])
    content = b"".join(lines[i + 1 :])
    return front_matter, content


def split_front_matter(content: bytes) -> tuple[FrontMatter, bytes]:
    front_matter, content = _split_front_matter_without_parsing_yaml(content)
    return FrontMatter.parse_obj(yaml.safe_load(front_matter) or {}), content


def merge_front_matter(front_matter: FrontMatter, content: bytes) -> bytes:
    if not front_matter:
        return content
    return b"\n".join(
        [
            _separator,
            yaml.safe_dump(front_matter.dict()).rstrip().encode(),
            _separator,
            content,
        ]
    )



import importlib.resources
import os
import pathlib
from logging import getLogger
from typing import Any, Optional

import yaml

from competitive_verifier import git, github

from .. import config as conf
from .type import SiteRenderConfig

_RESOURCE_PACKAGE = "competitive_verifier_resources"
_CONFIG_YML_PATH = "_config.yml"

COMPETITIVE_VERIFY_DOCS_CONFIG_YML = "COMPETITIVE_VERIFY_DOCS_CONFIG_YML"

logger = getLogger(__name__)


def _get_default_docs_dir() -> pathlib.Path:
    default_docs_dir = conf.config_dir / "docs"
    oj_verify_docs_dir = pathlib.Path(".verify-helper/docs")
    if not default_docs_dir.exists() and oj_verify_docs_dir.exists():
        return oj_verify_docs_dir
    return default_docs_dir


def _load_user_render_config_yml(docs_dir: pathlib.Path) -> Optional[dict[str, Any]]:
    env_config_yml = os.getenv(COMPETITIVE_VERIFY_DOCS_CONFIG_YML)
    if env_config_yml:
        print(env_config_yml)
        try:
            user_config_yml = yaml.safe_load(env_config_yml)
            if isinstance(user_config_yml, dict):
                return user_config_yml  # type:ignore
            else:
                logger.error("failed to parse $COMPETITIVE_VERIFY_DOCS_CONFIG_YML")
        except Exception as e:
            logger.exception(
                "failed to parse $COMPETITIVE_VERIFY_DOCS_CONFIG_YML: %s",
                e,
            )

    user_config_yml_path = docs_dir / _CONFIG_YML_PATH
    if user_config_yml_path.exists():
        try:
            user_config_yml = yaml.safe_load(user_config_yml_path.read_bytes())
            if isinstance(user_config_yml, dict):
                return user_config_yml  # type:ignore
            else:
                logger.error("failed to parse %s: %s", user_config_yml_path.as_posix())
        except Exception as e:
            logger.exception(
                "failed to parse %s: %s", user_config_yml_path.as_posix(), e
            )


def load_render_config(
    *,
    docs_dir: Optional[pathlib.Path],
    destination_dir: pathlib.Path,
) -> SiteRenderConfig:
    # load default _config.yml
    default_config_yml = yaml.safe_load(
        (importlib.resources.files(_RESOURCE_PACKAGE) / _CONFIG_YML_PATH).read_bytes()
    )
    assert default_config_yml is not None

    config_yml: dict[str, Any] = default_config_yml
    if not docs_dir:
        docs_dir = _get_default_docs_dir()

    logger.info("docs_dir=%s", docs_dir.as_posix())
    user_config_yml = _load_user_render_config_yml(docs_dir)
    if user_config_yml:
        config_yml.update(user_config_yml)

    config_yml.setdefault("action_name", github.env.get_workflow_name())
    config_yml.setdefault("branch_name", github.env.get_ref_name())

    git_root = git.get_root_directory().resolve()
    basedir = pathlib.Path.cwd().relative_to(git_root).as_posix()
    if basedir.startswith("."):
        basedir = ""
    else:
        basedir = f"{basedir}/"
    config_yml.setdefault("basedir", basedir)

    static_dir = docs_dir / "static"
    index_md_path = docs_dir / "index.md"
    return SiteRenderConfig(
        config_yml=config_yml,
        static_dir=static_dir.resolve(),
        index_md=index_md_path.resolve(),
        destination_dir=destination_dir.resolve(),
    )



from pathlib import Path

from competitive_verifier import git
from competitive_verifier.documents.builder import load_static_files
from competitive_verifier.documents.type import SiteRenderConfig

STATIC_FILES_PATH = "src/competitive_verifier_resources/jekyll"
# For windows test
static_files = git.ls_files(STATIC_FILES_PATH)


def test_load_static_files():
    config = SiteRenderConfig(
        config_yml={"foo": 1},
        destination_dir=Path("foo"),
        index_md=Path(__file__),
        static_dir=Path(__file__),
    )
    d = load_static_files(config=config)
    files = {p.relative_to(STATIC_FILES_PATH) for p in static_files}
    files.add(Path("_config.yml"))

    for f in files:
        print(f.as_posix())

    expected = {"foo" / f for f in files}
    assert set(d.keys()) == expected



# Python Version: 3.x
import pathlib
from logging import getLogger
from typing import Any, Optional, TypedDict

import toml

logger = getLogger(__name__)


class OjVerifyConfig(TypedDict):
    languages: dict[str, Any]


_loaded_config: Optional[OjVerifyConfig] = None


def _load(config_path: pathlib.Path) -> Optional[OjVerifyConfig]:
    if config_path.exists():
        return OjVerifyConfig(**toml.load(str(config_path)))
    return None


def set_config_path(config_path: pathlib.Path) -> None:
    global _loaded_config  # pylint: disable=invalid-name
    assert _loaded_config is None
    _loaded_config = _load(config_path)
    if _loaded_config is None:
        _loaded_config = OjVerifyConfig(languages={})
        logger.info("no config file")
    else:
        logger.info("config file loaded: %s: %s", str(config_path), _loaded_config)


def get_config() -> OjVerifyConfig:
    global _loaded_config  # pylint: disable=invalid-name
    if _loaded_config is None:
        # Use default config
        _loaded_config = OjVerifyConfig(languages={})
    return _loaded_config



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src',
 'competitive_verifier_oj_clone': 'src/competitive_verifier_oj_clone',
 'competitive_verifier_oj_clone.languages': 'src/competitive_verifier_oj_clone/languages',
 'competitive_verifier_resources': 'src/competitive_verifier_resources'}

packages = \
['competitive_verifier',
 'competitive_verifier.check',
 'competitive_verifier.console',
 'competitive_verifier.documents',
 'competitive_verifier.download',
 'competitive_verifier.merge_input',
 'competitive_verifier.merge_result',
 'competitive_verifier.migrate',
 'competitive_verifier.models',
 'competitive_verifier.oj_resolve',
 'competitive_verifier.verify',
 'competitive_verifier_oj_clone',
 'competitive_verifier_oj_clone.languages',
 'competitive_verifier_resources']

package_data = \
{'': ['*'],
 'competitive_verifier_resources': ['jekyll/*',
                                    'jekyll/_includes/*',
                                    'jekyll/_includes/highlight/*',
                                    'jekyll/_includes/mathjax/*',
                                    'jekyll/_layouts/*',
                                    'jekyll/assets/css/*',
                                    'jekyll/assets/js/*']}

install_requires = \
['colorama>=0.4.6,<0.5.0',
 'colorlog>=6.7.0,<7.0.0',
 'importlab>=0.8,<0.9',
 'online-judge-tools==11.5.1',
 'pydantic>=1.10.2,<2.0.0',
 'pyyaml>=6.0,<7.0']

entry_points = \
{'console_scripts': ['competitive-verifier = '
                     'competitive_verifier.console.app:main']}

setup_kwargs = {
    'name': 'competitive-verifier',
    'version': '1.3.0',
    'description': 'Verifier for libraries of competitive programming',
    'long_description': '# competitive-verifier\n\n[![Actions Status](https://github.com/competitive-verifier/competitive-verifier/workflows/verify/badge.svg)](https://github.com/competitive-verifier/competitive-verifier/actions) [![GitHub Pages](https://img.shields.io/static/v1?label=GitHub+Pages&message=+&color=brightgreen&logo=github)](https://competitive-verifier.github.io/competitive-verifier)\n[![PyPI](https://img.shields.io/pypi/v/competitive-verifier)](https://pypi.org/project/competitive-verifier/)\n\nThe library is inspired by [online-judge-tools/verification-helper](https://github.com/online-judge-tools/verification-helper).\n\nIf you want more info, see [DESIGN.md](DESIGN.md).\n\n## Get started\n\n### GitHub Actions\n\nSee [GitHub Pages](https://competitive-verifier.github.io/competitive-verifier/installer.html).\n[日本語](https://competitive-verifier.github.io/competitive-verifier/installer.ja.html)\n\n### Install(local)\n\nNeeds Python 3.9 or greater.\n\n```sh\npip install competitive-verifier\n```\n\nOr\n\n```sh\npip install git+https://github.com/competitive-verifier/competitive-verifier.git@latest\n```\n\n#### Migrate from verification-helper\n\nRun this script.\n\n```sh\ncompetitive-verifier migrate\n```\n\n## Development\n\n```sh\npip install -U poetry\npoetry install\n\n# test\npoetry run pytest\n\n# format\npoetry run poe format\n```',
    'author': 'kzrnm',
    'author_email': 'gengesa@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/competitive-verifier/competitive-verifier',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import hashlib
import os
import pathlib
import shutil
import sys
from contextlib import nullcontext
from logging import getLogger
from typing import Optional

import onlinejudge._implementation.utils
import onlinejudge.utils
import onlinejudge_command.main
import onlinejudge_command.subcommand.download
import onlinejudge_command.subcommand.test
from onlinejudge.service.library_checker import LibraryCheckerProblem
from onlinejudge.service.yukicoder import YukicoderService
from onlinejudge.type import NotLoggedInError

import competitive_verifier.config
import competitive_verifier.exec
from competitive_verifier import log

_oj_cache_dir = competitive_verifier.config.cache_dir.resolve() / "online-judge-tools"
_problem_cache_dir = competitive_verifier.config.cache_dir / "problems"
onlinejudge._implementation.utils.user_cache_dir = _oj_cache_dir
logger = getLogger(__name__)

checker_exe_path = "checker.exe" if sys.platform == "win32" else "checker"


def get_cache_directory() -> pathlib.Path:
    return _oj_cache_dir


def get_directory(url: str) -> pathlib.Path:
    return _problem_cache_dir / hashlib.md5(url.encode()).hexdigest()


def is_yukicoder(url: str) -> bool:
    return YukicoderService.from_url(url) is not None


def get_checker_problem(url: str) -> Optional[LibraryCheckerProblem]:
    return LibraryCheckerProblem.from_url(url)


def get_checker_path(url: str) -> Optional[pathlib.Path]:
    checker_problem = get_checker_problem(url)
    if checker_problem:
        problem_dir = (
            checker_problem._get_problem_directory_path()
        )  # pyright: reportPrivateUsage=false
        return problem_dir / checker_exe_path


def download(url: str, *, group_log: bool = False) -> bool:
    directory = get_directory(url)
    test_directory = directory / "test"

    logger.info("download[Start]: %s", url)
    if not (test_directory).exists() or list((test_directory).iterdir()) == []:
        logger.info("download[Run]: %s", url)

        if group_log:
            cm = log.group(f"download[Run]: {url}")
        else:
            cm = nullcontext()
        with cm:
            directory.mkdir(parents=True, exist_ok=True)
            # time.sleep(2)

            arg_list: list[str] = [
                "--cookie",
                str(get_cache_directory() / "cookie.txt"),
                "download",
                "--system",
                "-d",
                str(test_directory),
                "--silent",
                url,
            ]

            YUKICODER_TOKEN = os.environ.get("YUKICODER_TOKEN")
            if YUKICODER_TOKEN:
                arg_list += ["--yukicoder-token", YUKICODER_TOKEN]

            try:
                parser = onlinejudge_command.main.get_parser()  # type: ignore
                args = parser.parse_args(arg_list)  # type: ignore
                onlinejudge_command.subcommand.download.run(args)  # type: ignore

                checker_path = get_checker_path(url)
                if checker_path:
                    try:
                        shutil.copy2(checker_path, directory)
                    except Exception as e:
                        logger.exception("Failed to copy checker %s", e)
                        shutil.rmtree(directory)
                        return False
            except Exception as e:
                if isinstance(e, NotLoggedInError) and is_yukicoder(url):
                    logger.error("Required: $YUKICODER_TOKEN environment variable")
                else:
                    logger.exception("Failed to download", e)
                return False
    else:
        logger.info("download:already exists: %s", url)
    return True


def test(
    *,
    url: str,
    command: str,
    tle: float,
    error: Optional[float],
) -> bool:
    directory = get_directory(url)
    test_directory = directory / "test"

    arg_list: list[str] = [
        "--cookie",
        str(get_cache_directory() / "cookie.txt"),
        "test",
        "-c",
        command,
        "-d",
        str(test_directory),
        "--print-input",
        "--tle",
        str(tle),
    ]

    checker_path = directory / checker_exe_path
    if checker_path.exists():
        arg_list += ["--judge-command", str(checker_path)]
    if error:
        arg_list += ["-e", str(error)]

    parser = onlinejudge_command.main.get_parser()  # type: ignore
    args = parser.parse_args(arg_list)  # type: ignore
    return bool(onlinejudge_command.subcommand.test.run(args))  # type: ignore



import os
import pathlib
import uuid
from contextlib import contextmanager
from typing import Optional, TextIO


def _optional_path(strpath: Optional[str]) -> Optional[pathlib.Path]:
    return pathlib.Path(strpath) if strpath else None


class env:
    @classmethod
    def is_in_github_actions(cls) -> bool:
        return os.getenv("GITHUB_ACTIONS") == "true"

    @classmethod
    def is_enable_debug(cls) -> bool:
        return os.getenv("GITHUB_ACTIONS") == "1"

    @classmethod
    def get_ref_name(cls) -> Optional[str]:
        return os.getenv("GITHUB_REF_NAME")

    @classmethod
    def get_api_token(cls) -> Optional[str]:
        return os.getenv("GITHUB_TOKEN")

    @classmethod
    def get_event_name(cls) -> Optional[str]:
        return os.getenv("GITHUB_EVENT_NAME")

    @classmethod
    def get_api_url(cls) -> Optional[str]:
        return os.getenv("GITHUB_API_URL")

    @classmethod
    def get_repository(cls) -> Optional[str]:
        return os.getenv("GITHUB_REPOSITORY")

    @classmethod
    def get_workflow_name(cls) -> Optional[str]:
        return os.getenv("GITHUB_WORKFLOW")

    @classmethod
    def get_output_path(cls) -> Optional[pathlib.Path]:
        strpath = os.getenv("GITHUB_OUTPUT")
        return _optional_path(strpath)

    @classmethod
    def get_workspace_path(cls) -> Optional[pathlib.Path]:
        strpath = os.getenv("GITHUB_WORKSPACE")
        return _optional_path(strpath)

    @classmethod
    def get_step_summary_path(cls) -> Optional[pathlib.Path]:
        strpath = os.getenv("GITHUB_STEP_SUMMARY")
        return _optional_path(strpath)


def print_debug(
    message: str,
    *,
    title: Optional[str] = None,
    file: Optional[str] = None,
    col: Optional[int] = None,
    endColumn: Optional[int] = None,
    line: Optional[int] = None,
    endLine: Optional[int] = None,
    force: bool = False,
    stream: Optional[TextIO] = None,
) -> None:
    _print_github(
        "debug",
        message,
        title=title,
        file=file,
        col=col,
        endColumn=endColumn,
        line=line,
        endLine=endLine,
        force=force,
        stream=stream,
    )


def print_warning(
    message: str,
    *,
    title: Optional[str] = None,
    file: Optional[str] = None,
    col: Optional[int] = None,
    endColumn: Optional[int] = None,
    line: Optional[int] = None,
    endLine: Optional[int] = None,
    force: bool = False,
    stream: Optional[TextIO] = None,
) -> None:
    _print_github(
        "warning",
        message,
        title=title,
        file=file,
        col=col,
        endColumn=endColumn,
        line=line,
        endLine=endLine,
        force=force,
        stream=stream,
    )


def print_error(
    message: str,
    *,
    title: Optional[str] = None,
    file: Optional[str] = None,
    col: Optional[int] = None,
    endColumn: Optional[int] = None,
    line: Optional[int] = None,
    endLine: Optional[int] = None,
    force: bool = False,
    stream: Optional[TextIO] = None,
) -> None:
    _print_github(
        "error",
        message,
        title=title,
        file=file,
        col=col,
        endColumn=endColumn,
        line=line,
        endLine=endLine,
        force=force,
        stream=stream,
    )


def _print_github(
    command: str,
    message: str,
    *,
    title: Optional[str] = None,
    file: Optional[str] = None,
    col: Optional[int] = None,
    endColumn: Optional[int] = None,
    line: Optional[int] = None,
    endLine: Optional[int] = None,
    force: bool = False,
    stream: Optional[TextIO] = None,
) -> None:
    """print Github Actions style message

    https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions

    Args:
        title: Custom title
        file: Filename
        col: Column number, starting at 1
        endColumn: End column number
        line: Line number, starting at 1
        endLine: End line number
    """
    annotation = ",".join(
        f"{tup[0]}={tup[1]}"
        for tup in (
            ("title", title),
            ("file", file),
            ("col", col),
            ("endColumn", endColumn),
            ("line", line),
            ("endLine", endLine),
        )
        if tup[1] is not None
    )

    if force or env.is_in_github_actions():
        print(f"::{command} {annotation}::{message}", file=stream)


def begin_group(title: str, *, stream: Optional[TextIO] = None):
    print(f"::group::{title}", file=stream)


def end_group(*, stream: Optional[TextIO] = None):
    print("::endgroup::", file=stream)


def set_output(name: str, value: str) -> bool:
    path = env.get_output_path()
    if path and path.exists():
        delimiter = "outputdelimiter_" + str(uuid.uuid4())
        with open(path, mode="a", encoding="utf-8") as fp:
            fp.write(name)
            fp.write("<<")
            fp.write(delimiter + "\n")
            fp.write(value + "\n")
            fp.write(delimiter + "\n")
        return True
    return False


@contextmanager
def group(title: str, *, stream: Optional[TextIO] = None):
    try:
        begin_group(title, stream=stream)
        yield
    finally:
        end_group(stream=stream)



# flake8: noqa E501
import pathlib
import sys
from dataclasses import dataclass
from typing import Any, Sequence
from unittest import mock

import pytest
from pydantic import BaseModel

import competitive_verifier.models
from competitive_verifier.models import (
    CommandVerification,
    ConstVerification,
    ProblemVerification,
    ResultStatus,
    Verification,
)


@dataclass
class DataVerificationParams:
    default_tle: float


test_command_union_json_params = [  # type: ignore
    (
        ConstVerification(status=ResultStatus.SUCCESS),
        '{"type": "const","status":"success"}',
        '{"type": "const", "status": "success"}',
    ),
    (
        ConstVerification(status=ResultStatus.FAILURE),
        '{"type": "const","status":"failure"}',
        '{"type": "const", "status": "failure"}',
    ),
    (
        ConstVerification(status=ResultStatus.SKIPPED),
        '{"type": "const","status":"skipped"}',
        '{"type": "const", "status": "skipped"}',
    ),
    (
        ConstVerification(status=ResultStatus.SUCCESS),
        '{"type": "const","status":"success"}',
        '{"type": "const", "status": "success"}',
    ),
    (
        CommandVerification(command="ls ~"),
        '{"type":"command","command":"ls ~"}',
        '{"type": "command", "command": "ls ~", "compile": null}',
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        '{"type": "command", "compile": "cat LICENSE", "command": "ls ~"}',
        '{"type": "command", "command": "ls ~", "compile": "cat LICENSE"}',
    ),
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        '{"type":"problem","problem":"https://example.com","command":"ls ~"}',
        '{"type": "problem", "command": "ls ~", "compile": null, "problem": "https://example.com", "error": null, "tle": null}',
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        '{"type": "problem",  "problem":"https://example.com", "compile": "cat LICENSE", "command": "ls ~"}',
        '{"type": "problem", "command": "ls ~", "compile": "cat LICENSE", "problem": "https://example.com", "error": null, "tle": null}',
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        '{"type": "problem",  "problem":"https://example.com", "compile": "cat LICENSE", "error": 0.000001, "tle": 2, "command": "ls ~"}',
        '{"type": "problem", "command": "ls ~", "compile": "cat LICENSE", "problem": "https://example.com", "error": 1e-06, "tle": 2.0}',
    ),
]


@pytest.mark.parametrize("obj, raw_json, output_json", test_command_union_json_params)
def test_command_union_json(
    obj: Verification,
    raw_json: str,
    output_json: str,
):
    class VerificationUnion(BaseModel):
        __root__: Verification

    assert obj == type(obj).parse_raw(raw_json)
    assert obj.json() == output_json

    assert obj == VerificationUnion.parse_raw(raw_json).__root__


def mock_exec_command():
    return mock.patch("subprocess.run")


def test_const_verification():
    obj = ConstVerification(status=ResultStatus.SUCCESS)
    with mock_exec_command() as patch:
        assert obj.run() == ResultStatus.SUCCESS
        assert obj.run_compile_command()
        patch.assert_not_called()


test_run_params = [  # type: ignore
    (
        CommandVerification(command="ls ~"),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
]


@pytest.mark.parametrize("obj, args, kwargs", test_run_params)
def test_run(obj: Verification, args: Sequence[Any], kwargs: dict[str, Any]):
    with mock.patch(
        "onlinejudge._implementation.utils.user_cache_dir", pathlib.Path("/bar/baz")
    ), mock_exec_command() as patch:
        obj.run(
            DataVerificationParams(
                default_tle=22,
            ),
        )
        patch.assert_called_once_with(*args, **kwargs)


test_run_problem_command_params: list[tuple[ProblemVerification, dict[str, Any]]] = [
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        {
            "url": "https://example.com",
            "command": "ls ~",
            "tle": 22.0,
            "error": None,
        },
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        {
            "url": "https://example.com",
            "command": "ls ~",
            "tle": 22.0,
            "error": None,
        },
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        {
            "url": "https://example.com",
            "command": "ls ~",
            "tle": 2.0,
            "error": 1e-06,
        },
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://judge.yosupo.jp/problem/aplusb",
            error=1e-6,
            tle=2,
        ),
        {
            "url": "https://judge.yosupo.jp/problem/aplusb",
            "command": "ls ~",
            "tle": 2.0,
            "error": 1e-06,
        },
    ),
]


@pytest.mark.parametrize("obj, kwargs", test_run_problem_command_params)
def test_run_problem_command(obj: ProblemVerification, kwargs: dict[str, Any]):
    with mock.patch.object(
        competitive_verifier.models.verification.oj, "test"
    ) as patch:
        obj.run(
            DataVerificationParams(default_tle=22),
        )
        patch.assert_called_once_with(**kwargs)


test_run_compile_params = [  # type: ignore
    (
        CommandVerification(command="ls ~"),
        None,
        None,
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        ("cat LICENSE",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        None,
        None,
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        ("cat LICENSE",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        ("cat LICENSE",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
]


@pytest.mark.parametrize("obj, args, kwargs", test_run_compile_params)
def test_run_compile(obj: Verification, args: Sequence[Any], kwargs: dict[str, Any]):
    with mock_exec_command() as patch:
        obj.run_compile_command(
            DataVerificationParams(
                default_tle=22,
            ),
        )
        if args is None:
            patch.assert_not_called()
        else:
            patch.assert_called_once_with(*args, **kwargs)


test_params_run_params = [  # type: ignore
    (ConstVerification(status=ResultStatus.SUCCESS), None),
    (CommandVerification(command="true"), None),
    (
        ProblemVerification(command="true", problem="http://example.com"),
        "ProblemVerification.run requires VerificationParams",
    ),
]


@pytest.mark.parametrize("obj, error_message", test_params_run_params)
def test_params_run(obj: Verification, error_message: str):
    with mock_exec_command():
        if error_message:
            with pytest.raises(ValueError) as e:
                obj.run()
            assert e.value.args == (error_message,)
        else:
            obj.run()



import argparse
import pathlib
import sys
from functools import reduce
from logging import getLogger
from typing import Iterable, Optional

from competitive_verifier.log import configure_stderr_logging
from competitive_verifier.models.file import VerificationInput

logger = getLogger(__name__)


def merge(inputs: Iterable[VerificationInput]) -> VerificationInput:
    return reduce(lambda a, b: a.merge(b), inputs)


def run_impl(
    *verify_files_json: pathlib.Path,
) -> VerificationInput:
    configure_stderr_logging()
    result = merge(map(VerificationInput.parse_file_relative, verify_files_json))
    return result


def run(args: argparse.Namespace) -> bool:
    merged = run_impl(*args.verify_files_json)
    print(merged.json(exclude_none=True))
    return True


def argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    parser.add_argument(
        "verify_files_json",
        nargs="+",
        help="verify_files.json files",
        type=pathlib.Path,
    )
    return parser


def main(args: Optional[list[str]] = None) -> None:
    try:
        parsed = argument(argparse.ArgumentParser()).parse_args(args)
        if not run(parsed):
            sys.exit(1)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(2)


if __name__ == "__main__":
    main()
